;
; Program PICSMT pro mereni teploty z cidla SMT 160 pomoci mikroprocesoru PIC
; verze 1.0 z 25.1.2003 (Tomas OK1VCF)
; verze 1.1 z 27.1.2003 (Tomas OK1VCF) optimalizace kodu (usetreno 73 WORDS)
; Cidlo se pripoji na pin RB4, RS-232 prijima na pinu RA4, posila po pinu RA3
;  je-li na pinu RA2 log. 1, posila PIC po RS-232 zmerena data, je-li tam
;  log. 0 ocekava komunikaci po seriove lince (nastaveni parametru)
; Zmerena data posila po RS-232 rychlosti 9k6 v APRS WX formatu:
;  !5000.00N/01400.00E_000/000g000tFFFXPIC SMT temp:CCC
;  kde FFF je teplota ve Fahrenheitech a CCC ve stupnich Celsia
; Po zapnuti posle verzi sw, CR, LF a pak podle stavu pinu RA2 pokracuje
;  v provadeni programu.
; V EEPROM bytu na adrese 08h je ulozena konstanta pro casovani RS-232,
;  pro 10MHz oscilator by to mlo bt 52h, pro 4MHz oscilator 1Eh.
;  podle vzorce: ((ClockSpeed/4/SerialSpeed)-14)/3
;  Pokud PICSMT nebude reagovat na prikazy (^V ^U) je treba hodnotu zmenit.
;  Od uvedene se muze lisit az o +/- 4h (dle zkusenosti)
; Prikazy rezimu konfigurace jsou ve formatu ESCape nasledovany znakem:
;  ^V posle verzi sw
;  ^U posle obsah EEPROM, kteremu predchazi 1 BYTE s poctem znaku (v HEX)
;  ^P posle posit retezec z EEPROM, kteremu predchazi 1 BYTE s poctem znaku
;  ^D zapise do EEPROM nasledujicich 64bytu (mezi jednotlivymi byty musi byt
;     min 200ms pauza)
;  ^O zapise posit retezec do EEPROM (mezi jednotlivymi byty musi byt
;     min 200ms pauza)
;  ^M provede mereni a posle data
;  Ale pozor, data musi mit spravny format, pro konfiguraci pouzivejte
;  pribaleny konfiguracni progam od Petra OK1MAB!
;
; Program je siren jako HamWare - tzn. pro radiomaterske vyuziti zdarma.
;
; Hodne radosti s WX na APRS preje Tomas OK1VCF!
;

	LIST P=PIC16F84, R=DEC
	include<p16f84.inc>

SMT_IN		EQU	004H		; INPUT PIN pro cidlo SMT 160 (RB4)

SERIAL_IN	EQU	004H		; RS232 INPUT (RA4)
SERIAL_OUT	EQU	003H		; RS232 OUTPUT (RA3)
KONFIG_IN	EQU	002H		; Konfig_SW (RA2)

		; RAM 0Ch-1Fh rezervovany pro dalsi vyvoj
		; RAM 20h-35h pouziva aritmetika
TeplFs		EQU	036H		; ASCII 100ky Farhenheitu
TeplFd		EQU	037H		; ASCII 10ky Farhenheitu
TeplFj		EQU	038H		; ASCII 1ky Farhenheitu
VlhkS		EQU	039H		; ASCII 10ky % Vlhkosti
VlhkJ		EQU	03AH		; ASCII 1ky % Vlhkosti
pod_nulou	EQU	03BH		; Indikace teploty pod nulou C
TeplH		EQU	03CH		; ASCII 10ky teploty C
TeplL		EQU	03DH		; ASCII jednotky teploty C
desetiny	EQU	03EH		; ASCII desetiny stupnu C

poc_mer		EQU	03FH		; registr poctu mereni 
horniSOUCET	EQU	040H		; horni bity scitaneho mereni
SOUCET		EQU	041H		; dolni bity scitaneho mereni
countH		EQU	042H		; horni bity teplotni smycky
countL		EQU	043H		; dolni bity teplotni smycky
stridaH		EQU	044H		; namerena strida SMT horni
stridaL		EQU	045H		; namerena strida SMT dolni
	
temp0		EQU	046H		; pomocny registr
temp1		EQU	047H		; pomocny registr
temp2		EQU	048H		; pomocny registr
delay_time	EQU	049H		; Casovaci konstanta RS232
serialDelayCount EQU	04AH		; Registr odectu casovani RS232
serialCount	EQU	04BH		; Pocet bitu pro RS232
serialData	EQU	04CH		; Byte pro zpracovani RS232kou
		; RAM 4Dh-4Fh rezervovany pro dalsi vyvoj

		ORG	0000H

Zacatek		
		CALL	serial_Init		; Nastavit seriovy I/O
		GOTO	Send_Version		; poslat verzi, pak do hl. prg

Version_Data:	
		ADDWF	PCL,F		; Relative addressing
		RETLW	050H		; 'P'
		RETLW	049H		; 'I'
		RETLW	043H		; 'C'
		RETLW	053H		; 'S'
		RETLW	04DH		; 'M'
		RETLW	054H		; 'T'
		RETLW	020H		; ' '
		RETLW	076H		; 'v'
		RETLW	031H		; '1'
		RETLW	02EH		; '.'
		RETLW	031H		; '1'
		RETLW	000H

	
Start_Mereni					; Mereni SMT-160 teploty 
		MOVLW	010H			; merime 16x
		MOVWF	poc_mer			; zde je pocitadlo
		CLRF	SOUCET			;
		CLRF	horniSOUCET		;
		MOVLW	020H			; mezeru
		MOVWF	pod_nulou		; ulozit sem

Mer_smycka	CALL	Mereni			;
		MOVF	stridaH, W		; 
		BCF	STATUS, 0		; vynulovat CY
		ADDWF	SOUCET, F		; pricist vysledek mereni
		BTFSC	STATUS,0		;
		INCF	horniSOUCET, F		; pretekl-li soucet, pricist to sem
		DECFSZ	poc_mer, F		; odecist pocet mereni
		GOTO 	Mer_smycka		; jeste budeme merit
	
 		; Spocitat prumer, tzn vydelit poctem mereni (16)
		; a dostaneme pocet pulstupnu (pocitano od -68C)
		; proto budeme delit 32 a nasobit 10 a dostaneme
		; pocet desetin stupnu (pocitano od -68C), pak
		; k vysledku pricist -680 desetin stupne a hotovo
		; 10/32 = 0.3125
		; 0.3125 = 7D 20 00 00 float (z fprep.exe)
		;   -680 = 88 AA 00 00 float (z fprep.exe)

		CLRF	AARGB0			; vynulovat
		MOVF	horniSOUCET, W		; vzit horni bity
		MOVWF	AARGB1			; a dat je sem
		MOVF	SOUCET, W		; vzit dolni bity
		MOVWF	AARGB2			; a dat je sem
		CALL	FLO2432			; prevest
		; 0.3125 = 7B 20 00 00 float (z fprep.exe)
		MOVLW	07DH			; Pripravit nasobeni cislem
		MOVWF	BEXP			; 0.078125
		MOVLW	020H			; 
		MOVWF	BARGB0			; 
		CLRF	BARGB1			; 
		CLRF	BARGB2			; 
		CALL	FPM32			; vynasobit
		;     -680 = 88 AA 00 00 float (z fprep.exe)
		MOVLW	088H			; Pripravit pricteni cisla
		MOVWF	BEXP			; -680
		MOVLW	0AAH			; 
		MOVWF	BARGB0			; 
		CLRF	BARGB1			; 
		CLRF	BARGB2			; 
		CALL	FPA32			; secist
		CALL	INT3224			; a prevest na int 
		MOVF	AARGB0, W		; a schovat pro 
		MOVWF	temp0			; prevod na 
		MOVF	AARGB1, W		; stupne 
		MOVWF	temp1			; Fahrenheita
		MOVF	AARGB2, W		; do docasnych
		MOVWF	temp2			; registru
		BTFSS	AARGB0, 7		; mame zaporne stupne C?
		GOTO	C_kladne		; ne, jsou kladne
		COMF	AARGB0, F		; ano, udelat 2kovy komplement
		COMF	AARGB1, F		; ano, udelat 2kovy komplement
		COMF	AARGB2, F		; ano, udelat 2kovy komplement
		MOVLW	02DH			; znamenko "-"
		MOVWF	pod_nulou		; ulozit sem

C_kladne					; 
		MOVF	AARGB1, W		; prevod na ASCII
		MOVWF	AARGB0			; je 16 bitovy
		MOVF	AARGB2, W		; prevod na ASCII
		MOVWF	AARGB1			; je 16 bitovy
		movlw	03H	         	; prevod na 3 mistne
		movwf	digit_count     	; desitkove cislo
		movlw	desetiny		; desetiny zacinaji zde
		CALL	int16ascii		; prevest na ascii

		MOVF	temp0, W		; obnovit int
		MOVWF	AARGB0			; desetin stupnu C
		MOVF	temp1, W		; z docasnych
		MOVWF	AARGB1			; registru
		MOVF	temp2, W		; pro prevod na
		MOVWF	AARGB2			; Fahrenheity
		CALL	FLO2432			; prevest na float

		; Prevest C na F dle vzorce $F = (1.8 * $C) + 32
		; pro pocet desetin C je to $F = (0.18 * $C) + 32
		;    32d = 84 00 00 00 float (z fprep.exe)
		;  0.18d = 7C 38 51 EC float (z fprep.exe)
		MOVLW	07CH			; Pripravit nasobeni cislem
		MOVWF	BEXP			; 0.18
		MOVLW	038H			; 
		MOVWF	BARGB0			; 
		MOVLW	051H			; 
		MOVWF	BARGB1			; 
		MOVLW	0ECH			; 
		MOVWF	BARGB2			; 
		CALL	FPM32			; vynasobit
		;   32d  = 84 00 00 00 float (z fprep.exe)
		MOVLW	084H			; Pripravit pricteni cisla
		MOVWF	BEXP			; 32
		CLRF	BARGB0			; 
		CLRF	BARGB1			; 
		CLRF	BARGB2			; 
		CALL	FPA32			; secist
		CALL	INT3224			; a prevest na int 
		
		CLRF	temp0			;
		BTFSS	AARGB0, 7		; mame zaporne stupne F?
		GOTO	F_kladne		; ne, jsou kladne
		COMF	AARGB0, F		; ano, udelat 2kovy komplement
		COMF	AARGB1, F		; ano, udelat 2kovy komplement
		COMF	AARGB2, F		; ano, udelat 2kovy komplement
		BSF	temp0, 0		; priznak zapornosti

F_kladne					;
		MOVF	AARGB2, W		; z 24bit int teploty (8bit)
		MOVWF	AARGB1			; pouze 16bit int pro prevod
		movlw	03H	         	; prevod na 3 mistne
		movwf	digit_count     	; desitkove cislo
		movlw	TeplFj			; jednotky zacinaji zde
		CALL	int16ascii		; prevest na ascii
		MOVLW	02DH			; znamenko "-"
		BTFSC	temp0, 0		; jsme <0F ?
		MOVWF	TeplFs			; ano, ulozit znamenko

		;odeslani APRS WX retezce ve formatu:
		;!5003.45N/01430.58E_000/000g000tFFFXPIC SMT temp:-cc.cC
		MOVLW	EE_WX_POS		; zde je v EEPROM offset WX souradnic
		CALL	EE_Send_232		; Odeslat souradnice

		MOVF	TeplFs, W		; Vzit znamenko/stovky F 
		CALL	serial_SEND_byte		; poslat ho
		MOVF	TeplFd, W		; Vzit desitky F
		CALL	serial_SEND_byte		; poslat je
		MOVF	TeplFj, W		; Vzit jednotky F
		CALL	serial_SEND_byte		; poslat je

		MOVLW	EE_WX_COMMENT		; zde je v EEPROM offset WX komentu 
		CALL	EE_Send_232		; Odeslat komentar

		MOVF	pod_nulou, W		; Vzit znamenko 
		CALL	serial_SEND_byte		; poslat ho
		MOVF	TeplH, W		; Vzit desitky stupnu
		CALL	serial_SEND_byte		; poslat je
		MOVF	TeplL, W		; Vzit jednotky stupnu
		CALL	serial_SEND_byte		; poslat je
		MOVLW	02eH			; desetinna tecka
		CALL	serial_SEND_byte		; poslat ji
		MOVF	desetiny, W		; Vzit desitiny stupnu
		CALL	serial_SEND_byte		; poslat je
		MOVLW	043H			; C
		CALL	serial_SEND_byte		; poslat ji
		CALL	serialTXcr		; Odeslat CR po seriove lince
		GOTO	Main_Loop		; A cele znova


Mereni		; Rutina popsana Ondrejem Vitousem na HW serveru
		MOVLW	020H			; merime 108ms (425*32*8us)
		MOVWF	countL			; 435*32*8=1360=3520 +
		MOVLW	036H			; cyklus pro countL
		MOVWF	countH			; = 3620h

		CLRF	stridaL			; vynulovat citace
		CLRF	stridaH			; stridy
		BCF	STATUS, 2		; vynulovat Z
		
smycka9		nop				; 1 kazdy 256 cykl = 9us
smycka8		BTFSC	PORTB, SMT_IN		; 1/2
		INCF	stridaL, F		; 2	zvysit, pokud je SMT_IN log. 1
		BTFSC	STATUS, 2		; 3/4 
		INCF	stridaH, F		; 4	zvysit, pokud pretekl reg
			
		DECFSZ	countL, F		; 5/6
		GOTO	smycka9			; 6+1=7
		DECFSZ	countH, F		; 7/8
		GOTO	smycka8			; 7+1=8

		BCF	STATUS, 0		; 1 vynulovat CY
		RLF	stridaL, F		; 1 a odroloval
		RLF	stridaH, F		; 1 v ramci 
		RLF	stridaL, F		; 1 dvoubajtoveho
		RLF	stridaH, F		; 1 registru
		RLF	stridaL, F		; 1 32*, tzn mame
		RLF	stridaH, F		; 1 pocet pulstupnu

		RETURN				;

;*********************************************************************************************
;*********************************************************************************************
;*********************************************************************************************
;	RCS Header $Id: math16.inc 2.4 1997/02/11 16:58:49 F.J.Testa Exp $

;	$Revision: 2.4 $

;       MATH16 INCLUDE FILE
;
;       IMPORTANT NOTE: The math library routines can be used in a dedicated application on
;       an individual basis and memory allocation may be modified with the stipulation that
;       on the PIC17, P type registers must remain so since P type specific instructions
;	were used to realize some performance improvements.

;*********************************************************************************************
;
;       GENERAL MATH LIBRARY DEFINITIONS
;
;	general literal constants

;	define assembler constants

B0		equ	0
B1		equ	1
B2		equ	2
B3		equ	3
B4		equ	4
B5		equ	5
B6		equ	6
B7		equ	7

MSB		equ	7
LSB		equ	0


;     define commonly used bits

;     STATUS bit definitions

		#define	_C		STATUS,0
		#define	_Z		STATUS,2

;
;       general register variables (od 0x20H do 0x34H vcetne)
;

ACCB7           equ     0x20
ACCB6           equ     0x21
ACCB5           equ     0x22
ACCB4           equ     0x23
ACCB3           equ     0x24
ACCB2           equ     0x25
ACCB1           equ     0x26
ACCB0           equ     0x27
ACC             equ     0x27    ; most significant byte of contiguous 8 byte accumulator
;
SIGN            equ     0x29    ; save location for sign in MSB
;
TEMPB3          equ     0x30
TEMPB2          equ     0x31
TEMPB1          equ     0x32
TEMPB0          equ     0x33
TEMP            equ     0x33    ; temporary storage
;
;       binary operation arguments
;
AARGB7          equ     0x20
AARGB6          equ     0x21
AARGB5          equ     0x22
AARGB4          equ     0x23
AARGB3          equ     0x24
AARGB2          equ     0x25
AARGB1          equ     0x26
AARGB0          equ     0x27
AARG            equ     0x27    ; most significant byte of argument A
;
BARGB3          equ     0x2B
BARGB2          equ     0x2C
BARGB1          equ     0x2D
BARGB0          equ     0x2E
BARG            equ     0x2E    ; most significant byte of argument B
;
;       Note that AARG and ACC reference the same storage locations
;
;*********************************************************************************************
;
;       FIXED POINT SPECIFIC DEFINITIONS
;
;       remainder storage
;
REMB3           equ     0x20
REMB2           equ     0x21
REMB1           equ     0x22
REMB0           equ     0x23    ; most significant byte of remainder

LOOPCOUNT       equ     0x34    ; loop counter
;
;*********************************************************************************************
;
;       FLOATING POINT SPECIFIC DEFINITIONS
;
;       literal constants
;
EXPBIAS         equ     D'127'
;
;       biased exponents
;
EXP             equ     0x28    ; 8 bit biased exponent

AEXP            equ     0x28    ; 8 bit biased exponent for argument A

BEXP            equ     0x2F    ; 8 bit biased exponent for argument B
;
;       floating point library exception flags
;
FPFLAGS         equ     0x2A    ; floating point library exception flags
                
IOV             equ     0       ; bit0 = integer overflow flag

FOV             equ     1       ; bit1 = floating point overflow flag

FUN             equ     2       ; bit2 = floating point underflow flag

FDZ             equ     3       ; bit3 = floating point divide by zero flag

NAN		equ	4	; bit4 = not-a-number exception flag

DOM		equ	5	; bit5 = domain error exception flag

RND             equ     6       ; bit6 = floating point rounding flag, 0 = truncation
                                ; 1 = unbiased rounding to nearest LSB

SAT             equ     7       ; bit7 = floating point saturate flag, 0 = terminate on
                                ; exception without saturation, 1 = terminate on
                                ; exception with saturation to appropriate value

;**********************************************************************************************

;	RCS Header $Id: fp32.a16 2.8 1996/10/07 13:50:59 F.J.Testa Exp $

;	$Revision: 2.8 $

;       PIC16 32 BIT FLOATING POINT LIBRARY
;
;       Unary operations: both input and output are in AEXP,AARG
;
;       Binary operations: input in AEXP,AARG and BEXP,BARG with output in AEXP,AARG
;
;       All routines return WREG = 0x00 for successful completion, and WREG = 0xFF
;       for an error condition specified in FPFLAGS.
;
;       All timings are worst case cycle counts
;
;         Routine               Function
;
;       FLO2432         24 bit integer to 32 bit floating point conversion
;       FLO32
;
;               Timing:            RND
;                               0       1
;
;                       0       104     104
;                  SAT
;                       1       110     110
;
;       NRM3232   32 bit normalization of unnormalized 32 bit floating point numbers
;       NRM32
;
;               Timing:            RND
;                               0       1
;
;                       0       90      90
;                  SAT
;                       1       96      96
;
;
;       INT3224         32 bit floating point to 24 bit integer conversion
;       INT32
;
;
;               Timing:            RND
;                               0       1
;
;                       0       104      112
;                  SAT
;                       1       104      114
;
;
;       NRM4032 32 bit normalization of unnormalized 40 bit floating point numbers
;
;               Timing:            RND
;                               0       1
;
;                       0       112     128
;                  SAT
;                       1       112     135
;
;
;       FPA32           32 bit floating point add
;
;               Timing:            RND
;                               0       1
;
;                       0       251     265
;                  SAT
;                       1       251     271
;
;       FPS32           32 bit floating point subtract
;
;               Timing:            RND
;                               0       1
;
;                       0       253     267
;                  SAT
;                       1       253     273
;
;       FPM32           32 bit floating point multiply
;
;               Timing:            RND
;                               0       1
;
;                       0       574     588
;                  SAT
;                       1       574     591
;
;**********************************************************************************************
;**********************************************************************************************
;
;       32 bit floating point representation
;
;       EXPONENT        8 bit biased exponent
;
;                       It is important to note that the use of biased exponents produces
;                       a unique representation of a floating point 0, given by
;                       EXP = HIGHBYTE = MIDBYTE = LOWBYTE = 0x00, with 0 being
;                       the only number with EXP = 0.
;
;       HIGHBYTE        8 bit most significant byte of fraction in sign-magnitude representation,
;                       with SIGN = MSB, implicit MSB = 1 and radix point to the right of MSB
;
;       MIDBYTE         8 bit middle significant byte of sign-magnitude fraction
;
;       LOWBYTE         8 bit least significant byte of sign-magnitude fraction
;
;       EXPONENT        HIGHBYTE        MIDBYTE         LOWBYTE
;
;       xxxxxxxx        S.xxxxxxx       xxxxxxxx        xxxxxxxx
;
;                        |
;                      RADIX
;                      POINT
;
;
;**********************************************************************************************
;**********************************************************************************************

;       Integer to float conversion

;       Input:  24 bit 2's complement integer right justified in AARGB0, AARGB1, AARGB2

;       Use:    CALL    FLO2432 or      CALL    FLO32

;       Output: 32 bit floating point number in AEXP, AARGB0, AARGB1, AARGB2

;       Result: AARG  <--  FLOAT( AARG )

;       Max Timing:     14+90 = 104 clks                SAT = 0
;                       14+96 = 110 clks                SAT = 1

;       Min Timing:     6+28 = 34 clks                  AARG = 0
;                       6+18 = 24 clks

;       PM: 14+38 = 52                                  DM: 7

;----------------------------------------------------------------------------------------------

FLO2432
FLO32           MOVLW           D'23'+EXPBIAS		; initialize exponent and add bias
                MOVWF           EXP
                CLRF            SIGN
                BTFSS           AARGB0,MSB		; test sign
                GOTO            NRM3232
                COMF            AARGB2,F                ; if < 0, negate and set MSB in SIGN
                COMF            AARGB1,F
                COMF            AARGB0,F
                INCF            AARGB2,F
                BTFSC           _Z
                INCF            AARGB1,F
                BTFSC           _Z
                INCF            AARGB0,F
                BSF             SIGN,MSB

;**********************************************************************************************

;       Normalization routine

;       Input:  32 bit unnormalized floating point number in AEXP, AARGB0, AARGB1,
;               AARGB2, with sign in SIGN,MSB

;       Use:    CALL    NRM3232 or      CALL    NRM32

;       Output: 32 bit normalized floating point number in AEXP, AARGB0, AARGB1, AARGB2

;       Result: AARG  <--  NORMALIZE( AARG )

;       Max Timing:     21+6+7*8+7 = 90 clks            SAT = 0
;                       21+6+7*8+1+12 = 96 clks SAT = 1

;       Min Timing:     22+6 = 28 clks                  AARG = 0
;                       5+9+4 = 18 clks

;       PM: 38                                          DM: 7

;----------------------------------------------------------------------------------------------

NRM3232

NRM32           CLRF            TEMP			; clear exponent decrement
                MOVF            AARGB0,W		; test if highbyte=0
                BTFSS           _Z
                GOTO            NORM3232
                MOVF            AARGB1,W		; if so, shift 8 bits by move
                MOVWF           AARGB0
                MOVF            AARGB2,W
                MOVWF           AARGB1
                CLRF            AARGB2
                BSF             TEMP,3                  ; increase decrement by 8

                MOVF            AARGB0,W		; test if highbyte=0
                BTFSS           _Z
                GOTO            NORM3232
                MOVF            AARGB1,W		; if so, shift 8 bits by move
                MOVWF           AARGB0
                CLRF            AARGB1
                BCF             TEMP,3                  ; increase decrement by 8
                BSF             TEMP,4
        
                MOVF            AARGB0,W		; if highbyte=0, result=0
                BTFSC           _Z
                GOTO            RES032

NORM3232        MOVF            TEMP,W
                SUBWF           EXP,F
                BTFSS           _Z
                BTFSS           _C
                GOTO            SETFUN32

                BCF             _C                      ; clear carry bit

NORM3232A       BTFSC           AARGB0,MSB		; if MSB=1, normalization done
                GOTO            FIXSIGN32
                RLF             AARGB2,F                ; otherwise, shift left and 
                RLF             AARGB1,F                ; decrement EXP
                RLF             AARGB0,F
                DECFSZ          EXP,F
                GOTO            NORM3232A

                GOTO            SETFUN32                ; underflow if EXP=0

FIXSIGN32       BTFSS           SIGN,MSB
                BCF             AARGB0,MSB              ; clear explicit MSB if positive
                RETLW           0

RES032          CLRF            AARGB0                  ; result equals zero
                CLRF            AARGB1
                CLRF            AARGB2
		CLRF		AARGB3
                CLRF            EXP
                RETLW           0

;**********************************************************************************************

;       Normalization routine

;       Input:  40 bit unnormalized floating point number in AEXP, AARGB0, AARGB1,
;               AARGB2, AARGB3 with sign in SIGN,MSB

;       Use:    CALL    NRM4032

;       Output: 32 bit normalized floating point number in AEXP, AARGB0, AARGB1, AARGB2,
;               AARGB3

;       Result: AARG  <--  NORMALIZE( AARG )

;       Max Timing:     38+6*9+12+8 = 112 clks  RND = 0
;                       38+6*9+12+24 = 128 clks RND = 1, SAT = 0
;                       38+6*9+12+31 = 135 clks RND = 1, SAT = 1

;       Min Timing:     33+6 = 39 clks                  AARG = 0
;                       5+9+8 = 22 clks

;       PM: 66                                          DM: 8

;----------------------------------------------------------------------------------------------

NRM4032         CLRF            TEMP			; clear exponent decrement
                MOVF            AARGB0,W		; test if highbyte=0
                BTFSS           _Z
                GOTO            NORM4032
                MOVF            AARGB1,W		; if so, shift 8 bits by move
                MOVWF           AARGB0
                MOVF            AARGB2,W
                MOVWF           AARGB1
                MOVF            AARGB3,W
                MOVWF           AARGB2
                CLRF            AARGB3
                BSF             TEMP,3                  ; increase decrement by 8

                MOVF            AARGB0,W		; test if highbyte=0
                BTFSS           _Z
                GOTO            NORM4032
                MOVF            AARGB1,W		; if so, shift 8 bits by move
                MOVWF           AARGB0
                MOVF            AARGB2,W
                MOVWF           AARGB1
                CLRF            AARGB2
                BCF             TEMP,3                  ; increase decrement by 8
                BSF             TEMP,4
        
                MOVF            AARGB0,W		; test if highbyte=0
                BTFSS           _Z
                GOTO            NORM4032
                MOVF            AARGB1,W		; if so, shift 8 bits by move
                MOVWF           AARGB0
                CLRF            AARGB1
                BSF             TEMP,3                  ; increase decrement by 8
        
                MOVF            AARGB0,W		; if highbyte=0, result=0
                BTFSC           _Z
                GOTO            RES032

NORM4032        MOVF            TEMP,W
                SUBWF           EXP,F
                BTFSS           _Z
                BTFSS           _C
                GOTO            SETFUN32

                BCF             _C                      ; clear carry bit

NORM4032A       BTFSC           AARGB0,MSB		; if MSB=1, normalization done
                GOTO            NRMRND4032
                RLF             AARGB3,F                ; otherwise, shift left and 
                RLF             AARGB2,F                ; decrement EXP
                RLF             AARGB1,F
                RLF             AARGB0,F
                DECFSZ          EXP,F
                GOTO            NORM4032A

                GOTO            SETFUN32                ; underflow if EXP=0

NRMRND4032      BTFSC           FPFLAGS,RND
                BTFSS           AARGB2,LSB
                GOTO            FIXSIGN32
		BTFSS		AARGB3,MSB		; round if next bit is set
                GOTO            FIXSIGN32
		INCF		AARGB2,F
                BTFSC           _Z
                INCF            AARGB1,F
                BTFSC           _Z
                INCF            AARGB0,F

                BTFSS           _Z                      ; has rounding caused carryout?
                GOTO            FIXSIGN32
                RRF             AARGB0,F                ; if so, right shift
                RRF             AARGB1,F
                RRF             AARGB2,F
                INCF            EXP,F
                BTFSC           _Z                      ; check for overflow
                GOTO            SETFOV32
                GOTO            FIXSIGN32

;**********************************************************************************************
;**********************************************************************************************

;       Float to integer conversion

;       Input:  32 bit floating point number in AEXP, AARGB0, AARGB1, AARGB2

;       Use:    CALL    INT3224         or      CALL    INT32

;       Output: 24 bit 2's complement integer right justified in AARGB0, AARGB1, AARGB2

;       Result: AARG  <--  INT( AARG )

;       Max Timing:     40+6*7+6+16 = 104 clks		RND = 0
;                       40+6*7+6+24 = 112 clks		RND = 1, SAT = 0
;                       40+6*7+6+26 = 114 clks  	RND = 1, SAT = 1

;       Min Timing:     4 clks

;       PM: 82                                          DM: 6

;----------------------------------------------------------------------------------------------

INT3224
INT32
		MOVF		EXP,W			; test for zero argument
		BTFSC		_Z
		RETLW		0x00

		MOVF            AARGB0,W		; save sign in SIGN
                MOVWF           SIGN
                BSF             AARGB0,MSB		; make MSB explicit

                MOVLW           EXPBIAS+D'23'		; remove bias from EXP
                SUBWF           EXP,F
                BTFSS           EXP,MSB
                GOTO            SETIOV3224
		COMF		EXP,F
		INCF		EXP,F

                MOVLW           8                       ; do byte shift if EXP >= 8
                SUBWF           EXP,W
                BTFSS           _C
                GOTO            TSHIFT3224
                MOVWF           EXP
                RLF             AARGB2,F                ; rotate next bit for rounding
                MOVF            AARGB1,W
                MOVWF           AARGB2
                MOVF            AARGB0,W
                MOVWF           AARGB1
                CLRF            AARGB0

                MOVLW           8                       ; do another byte shift if EXP >= 8
                SUBWF           EXP,W
                BTFSS           _C
                GOTO            TSHIFT3224
                MOVWF           EXP
                RLF             AARGB2,F		; rotate next bit for rounding
                MOVF            AARGB1,W
                MOVWF           AARGB2
                CLRF            AARGB1

                MOVLW           8                       ; do another byte shift if EXP >= 8
                SUBWF           EXP,W
                BTFSS           _C
                GOTO            TSHIFT3224
                MOVWF           EXP
                RLF             AARGB2,F                ; rotate next bit for rounding
                CLRF            AARGB2
		MOVF		EXP,W
		BTFSS		_Z
		BCF		_C
		GOTO		SHIFT3224OK

TSHIFT3224      MOVF            EXP,W                   ; shift completed if EXP = 0
                BTFSC           _Z
                GOTO            SHIFT3224OK

SHIFT3224       BCF             _C
                RRF             AARGB0,F                ; right shift by EXP
                RRF             AARGB1,F
                RRF             AARGB2,F
                DECFSZ          EXP,F
                GOTO            SHIFT3224

SHIFT3224OK     BTFSC           FPFLAGS,RND
                BTFSS           AARGB2,LSB
                GOTO            INT3224OK
                BTFSS           _C
                GOTO            INT3224OK
                INCF            AARGB2,F
                BTFSC           _Z
                INCF            AARGB1,F
                BTFSC           _Z
                INCF            AARGB0,F
                BTFSC           AARGB0,MSB		; test for overflow
                GOTO            SETIOV3224

INT3224OK       BTFSS           SIGN,MSB                ; if sign bit set, negate               
                RETLW           0
                COMF            AARGB0,F
                COMF            AARGB1,F
                COMF            AARGB2,F
                INCF            AARGB2,F
                BTFSC           _Z
                INCF            AARGB1,F
                BTFSC           _Z
                INCF            AARGB0,F
                RETLW           0

IRES03224	CLRF            AARGB0			; integer result equals zero
                CLRF            AARGB1
                CLRF            AARGB2
                RETLW           0

SETIOV3224	BSF             FPFLAGS,IOV             ; set integer overflow flag
                BTFSS           FPFLAGS,SAT             ; test for saturation
                RETLW           0xFF                    ; return error code in WREG

                CLRF            AARGB0			; saturate to largest two's
                BTFSS           SIGN,MSB                ; complement 24 bit integer
                MOVLW           0xFF
                MOVWF           AARGB0			; SIGN = 0, 0x 7F FF FF
                MOVWF           AARGB1			; SIGN = 1, 0x 80 00 00
                MOVWF           AARGB2
                RLF             SIGN,F
                RRF             AARGB0,F
                RETLW           0xFF                    ; return error code in WREG

;**********************************************************************************************
;**********************************************************************************************

;       Floating Point Multiply

;       Input:  32 bit floating point number in AEXP, AARGB0, AARGB1, AARGB2
;               32 bit floating point number in BEXP, BARGB0, BARGB1, BARGB2

;       Use:    CALL    FPM32

;       Output: 32 bit floating point product in AEXP, AARGB0, AARGB1, AARGB2

;       Result: AARG  <--  AARG * BARG

;       Max Timing:     26+23*22+21+21 = 574 clks       RND = 0
;                       26+23*22+21+35 = 588 clks       RND = 1, SAT = 0
;                       26+23*22+21+38 = 591 clks       RND = 1, SAT = 1

;       Min Timing:     6+6 = 12 clks                   AARG * BARG = 0
;                       24+23*11+21+17 = 315 clks

;       PM: 94                                          DM: 14

;----------------------------------------------------------------------------------------------

FPM32           MOVF            AEXP,W                  ; test for zero arguments
                BTFSS           _Z
                MOVF            BEXP,W
                BTFSC           _Z
                GOTO            RES032

M32BNE0         MOVF            AARGB0,W
                XORWF           BARGB0,W
                MOVWF           SIGN                    ; save sign in SIGN

                MOVF            BEXP,W
                ADDWF           EXP,F
                MOVLW           EXPBIAS-1
                BTFSS           _C
                GOTO            MTUN32

                SUBWF           EXP,F
                BTFSC           _C
                GOTO            SETFOV32                ; set multiply overflow flag
                GOTO            MOK32

MTUN32          SUBWF           EXP,F
                BTFSS           _C
                GOTO            SETFUN32

MOK32		MOVF		AARGB0,W
		MOVWF		AARGB3
		MOVF		AARGB1,W
		MOVWF		AARGB4
		MOVF		AARGB2,W
		MOVWF		AARGB5
		BSF             AARGB3,MSB              ; make argument MSB's explicit
                BSF             BARGB0,MSB
                BCF             _C
                CLRF            AARGB0			; clear initial partial product
                CLRF            AARGB1
                CLRF            AARGB2
                MOVLW           D'24'
                MOVWF           TEMP                    ; initialize counter

MLOOP32         BTFSS           AARGB5,LSB              ; test next bit
                GOTO            MNOADD32

MADD32          MOVF            BARGB2,W
                ADDWF           AARGB2,F
                MOVF            BARGB1,W
                BTFSC           _C
                INCFSZ          BARGB1,W
                ADDWF           AARGB1,F

                MOVF            BARGB0,W
                BTFSC           _C
                INCFSZ          BARGB0,W
                ADDWF           AARGB0,F

MNOADD32        RRF             AARGB0,F
                RRF             AARGB1,F
                RRF             AARGB2,F
                RRF             AARGB3,F
                RRF             AARGB4,F
                RRF             AARGB5,F
                BCF             _C
                DECFSZ          TEMP,F
                GOTO            MLOOP32

                BTFSC           AARGB0,MSB               ; check for postnormalization
                GOTO            MROUND32
                RLF             AARGB3,F
                RLF             AARGB2,F
                RLF             AARGB1,F
                RLF             AARGB0,F
                DECF            EXP,F

MROUND32        BTFSC           FPFLAGS,RND
                BTFSS           AARGB2,LSB
                GOTO            MUL32OK
		BTFSS		AARGB3,MSB
                GOTO            MUL32OK
		INCF		AARGB2,F
                BTFSC           _Z
                INCF            AARGB1,F
                BTFSC           _Z
                INCF            AARGB0,F

                BTFSS           _Z                      ; has rounding caused carryout?
                GOTO            MUL32OK
                RRF             AARGB0,F                ; if so, right shift
                RRF             AARGB1,F
                RRF             AARGB2,F
                INCF            EXP,F
                BTFSC           _Z                      ; check for overflow
                GOTO            SETFOV32

MUL32OK         BTFSS           SIGN,MSB
                BCF             AARGB0,MSB		; clear explicit MSB if positive

                RETLW           0  

SETFOV32        BSF             FPFLAGS,FOV             ; set floating point underflag
                BTFSS           FPFLAGS,SAT             ; test for saturation
                RETLW           0xFF                    ; return error code in WREG

                MOVLW           0xFF
                MOVWF           AEXP                    ; saturate to largest floating
                MOVWF           AARGB0                  ; point number = 0x FF 7F FF FF
                MOVWF           AARGB1                  ; modulo the appropriate sign bit
                MOVWF           AARGB2
                RLF             SIGN,F
                RRF             AARGB0,F
                RETLW           0xFF                    ; return error code in WREG

;**********************************************************************************************

SETFUN32        BSF             FPFLAGS,FUN             ; set floating point underflag
                BTFSS           FPFLAGS,SAT             ; test for saturation
                RETLW           0xFF                    ; return error code in WREG

                MOVLW           0x01                    ; saturate to smallest floating
                MOVWF           AEXP                    ; point number = 0x 01 00 00 00
                CLRF            AARGB0                  ; modulo the appropriate sign bit
                CLRF            AARGB1
                CLRF            AARGB2
                RLF             SIGN,F
                RRF             AARGB0,F
                RETLW           0xFF                    ; return error code in WREG

;**********************************************************************************************
;**********************************************************************************************

;       Floating Point Subtract

;       Input:  32 bit floating point number in AEXP, AARGB0, AARGB1, AARGB2
;               32 bit floating point number in BEXP, BARGB0, BARGB1, BARGB2

;       Use:    CALL FPS32

;       Output: 32 bit floating point sum in AEXP, AARGB0, AARGB1, AARGB2

;       Result: AARG  <--  AARG - BARG

;       Max Timing:     2+251 = 253 clks                RND = 0
;                       2+265 = 267 clks                RND = 1, SAT = 0
;                       2+271 = 273 clks                RND = 1, SAT = 1

;       Min Timing:     2+12 = 14 clks

;       PM: 2+146 = 148                         DM: 14

;----------------------------------------------------------------------------------------------

FPS32           MOVLW           0x80
                XORWF           BARGB0,F

;**********************************************************************************************

;       Floating Point Add

;       Input:  32 bit floating point number in AEXP, AARGB0, AARGB1, AARGB2
;               32 bit floating point number in BEXP, BARGB0, BARGB1, BARGB2

;       Use:    CALL FPA32

;       Output: 32 bit floating point sum in AEXP, AARGB0, AARGB1, AARGB2

;       Result: AARG  <--  AARG - BARG

;       Max Timing:     31+41+6*7+6+41+90 = 251 clks            RND = 0
;                       31+41+6*7+6+55+90 = 265 clks            RND = 1, SAT = 0
;                       31+41+6*7+6+55+96 = 271 clks            RND = 1, SAT = 1

;       Min Timing:     8+4 = 12 clks

;       PM: 146                                                 DM: 14

;----------------------------------------------------------------------------------------------

FPA32           MOVF            AARGB0,W                ; exclusive or of signs in TEMP
                XORWF           BARGB0,W
                MOVWF           TEMP

		CLRF		AARGB3			; clear extended byte
		CLRF		BARGB3

                MOVF            AEXP,W                  ; use AARG if AEXP >= BEXP
                SUBWF           BEXP,W
                BTFSS           _C
                GOTO            USEA32

                MOVF            BEXP,W                  ; use BARG if AEXP < BEXP
                MOVWF           AARGB5			; therefore, swap AARG and BARG
                MOVF            AEXP,W
                MOVWF           BEXP
                MOVF            AARGB5,W
                MOVWF           AEXP

                MOVF            BARGB0,W
                MOVWF           AARGB5
                MOVF            AARGB0,W
                MOVWF           BARGB0
                MOVF            AARGB5,W
                MOVWF           AARGB0

                MOVF            BARGB1,W
                MOVWF           AARGB5
                MOVF            AARGB1,W
                MOVWF           BARGB1
                MOVF            AARGB5,W
                MOVWF           AARGB1

                MOVF            BARGB2,W
                MOVWF           AARGB5
                MOVF            AARGB2,W
                MOVWF           BARGB2
                MOVF            AARGB5,W
                MOVWF           AARGB2

USEA32          MOVF            BEXP,W                  ; return AARG if BARG = 0
                BTFSC           _Z
                RETLW           0x00

                MOVF            AARGB0,W
                MOVWF           SIGN                    ; save sign in SIGN
                BSF             AARGB0,MSB              ; make MSB's explicit
                BSF             BARGB0,MSB

                MOVF            BEXP,W                  ; compute shift count in BEXP
                SUBWF           AEXP,W
                MOVWF           BEXP
                BTFSC           _Z
                GOTO            ALIGNED32

                MOVLW           8
                SUBWF           BEXP,W
                BTFSS           _C                      ; if BEXP >= 8, do byte shift
                GOTO            ALIGNB32
                MOVWF           BEXP
                MOVF            BARGB2,W		; keep for postnormalization
		MOVWF		BARGB3
                MOVF            BARGB1,W
		MOVWF		BARGB2
                MOVF            BARGB0,W
		MOVWF		BARGB1
                CLRF            BARGB0

                MOVLW           8
                SUBWF           BEXP,W
                BTFSS           _C                      ; if BEXP >= 8, do byte shift
                GOTO            ALIGNB32
                MOVWF           BEXP
                MOVF            BARGB2,W		; keep for postnormalization
		MOVWF		BARGB3
                MOVF            BARGB1,W
		MOVWF		BARGB2
                CLRF            BARGB1

                MOVLW           8
                SUBWF           BEXP,W
                BTFSS           _C                      ; if BEXP >= 8, BARG = 0 relative to AARG
                GOTO            ALIGNB32
                MOVF            SIGN,W
                MOVWF           AARGB0
                RETLW           0x00

ALIGNB32        MOVF            BEXP,W                  ; already aligned if BEXP = 0
                BTFSC           _Z
                GOTO            ALIGNED32

ALOOPB32        BCF             _C                      ; right shift by BEXP
                RRF             BARGB0,F
                RRF             BARGB1,F
		RRF		BARGB2,F
		RRF		BARGB3,F
                DECFSZ          BEXP,F
                GOTO            ALOOPB32

ALIGNED32       BTFSS           TEMP,MSB                ; negate if signs opposite
                GOTO            AOK32

		COMF		BARGB3,F
		COMF		BARGB2,F
                COMF            BARGB1,F
                COMF            BARGB0,F
                INCF            BARGB3,F
                BTFSC           _Z
                INCF            BARGB2,F
		BTFSC		_Z
		INCF		BARGB1,F
		BTFSC		_Z
		INCF		BARGB0,F

AOK32
                MOVF   		BARGB3,W
                ADDWF   	AARGB3,F
                MOVF            BARGB2,W
                BTFSC           _C
                INCFSZ          BARGB2,W
                ADDWF           AARGB2,F
                MOVF            BARGB1,W
                BTFSC           _C
                INCFSZ          BARGB1,W
                ADDWF           AARGB1,F
                MOVF            BARGB0,W
                BTFSC           _C
                INCFSZ          BARGB0,W
                ADDWF           AARGB0,F

                BTFSC           TEMP,MSB
                GOTO            ACOMP32
                BTFSS           _C
                GOTO            NRMRND4032

                RRF             AARGB0,F               ; shift right and increment EXP
                RRF             AARGB1,F
                RRF             AARGB2,F
		RRF		AARGB3,F
                INCFSZ          AEXP,F
                GOTO            NRMRND4032
                GOTO            SETFOV32

ACOMP32         BTFSC           _C
                GOTO            NRM4032			; normalize and fix sign

		COMF		AARGB3,F
                COMF            AARGB2,F		; negate, toggle sign bit and
                COMF            AARGB1,F		; then normalize
                COMF            AARGB0,F
                INCF            AARGB3,F
                BTFSC           _Z
                INCF            AARGB2,F
                BTFSC           _Z
                INCF            AARGB1,F
                BTFSC           _Z
                INCF            AARGB0,F

                MOVLW           0x80
                XORWF           SIGN,F
                GOTO            NRM32


		; AN670 32BIT NUMBER TO ASCII (ale zatim jen do cisla 9999!)

;****************************************************************************
;*		Floating Point to ASCII
;*
;* INPUT: 32 bit floating point number in AEXP, AARGB0, AARGB1, AARGB2
;*        For this example, the number must be between 0.000 and 9.999.
;*        (You can easily change the number range to suit your needs.)
;*
;* OUTPUT: ones, tenths, hundredths, thousandths (ASCII digit in each)
;*         
;*
;* USED: INDF,FSR,AARG,BARG,REMB,digit_count
;*
;* PROCEDURE: The floating point number in AARG is multiplied by 1000.
;*            Then the product is divided by 10 three times.  After each
;*            divide, the remainder is kept.  
;*
;*            After each digit is obtained, 30H is added to it to make it
;*            an ASCII representation of that number.  Then, the ASCII
;*            value is stored in register RAM at the locations specified
;*            in the "cblock."      
;*
;*	Note: The ASCII decimal point is not generated by this routine.
;*            You must output the decimal point in the correct decimal
;*            position for your application.  For this example, the
;*            decimal point is after the first digit: ones.
;*            
;* The following files are needed for this routine to function.
;*  p16c74a.inc		-- or any other midrange processor include file 
;*                         include the processor file in your "main" file
;*
;*  math16.inc          -- constant and variable definitions for 
;*                         AN575 floating point routines and
;*                         AN617 fixed point routines, both are used
;*                         in this float to ASCII routine
;*                         "include" this file in your "main" program
;*
;*  fxd26.a16 		-- fixed point 32/16 divide, included at the end
;*                         of this routine.
;*
;*  fp32.a16		-- 32 bit float to 32 bit integer conversion 
;*                         included at the end of this program. 
;* 
;****************************************************************************


;RAM Register Definitions       Your "main" program must have a "cblock"
                                ;directive with a RAM address so the 
                                ;following "cblock" will be located in RAM

digit_count	EQU	035H	;counter used to cycle through each digit 

int16ascii			; Prevod 16bitove binarni hodnoty na ascii
	; nasledujici tri radky udelat pred volanim rutiny, tak aby rutina dostala 
	; adresu kam ukladat vysledek ve 'W' a pocet mist byl v digit_count
	;movlw	SIG_FIG         ;load counter with the number of
	;movwf	digit_count     ;significant figures the decimal number	
	;movlw	last_digit	;

	movwf	FSR		;pointer = address of smallest digit

flo_asclp
	clrf	BARGB0		;Make the divisor 10.  
	movlw	d'10'
	movwf	BARGB1

	call	FXD1616U	;divide (16 bit fixed) / 10 (to get remainder)
	
	movf	REMB1,w		;put remainder in w register
	movwf	INDF		;put number into appropriate digit position

	movlw	0x30
	addwf	INDF,f		;add 30H to decimal number to convert to ASCII

	decf	FSR,f		;move pointer to next digit 	

	decfsz	digit_count,f
	goto	flo_asclp

	return


;**********************************************************************************************
;**********************************************************************************************

;	RCS Header $Id: fxd66.a16 2.4 1997/02/27 01:20:22 F.J.Testa Exp $

;	$Revision: 2.4 $

;       16/16 PIC16 FIXED POINT DIVIDE ROUTINES
;
;       Input:  fixed point arguments in AARG and BARG
;
;       Output: quotient AARG/BARG followed by remainder in REM
;
;       All timings are worst case cycle counts
;
;       It is useful to note that the additional unsigned routines requiring a non-power of two
;       argument can be called in a signed divide application where it is known that the
;       respective argument is nonnegative, thereby offering some improvement in
;       performance.
;
;         Routine            Clocks     Function
;
;       FXD1616U    373 16 bit/16 bit -> 16.16 unsigned fixed point divide
;
;       The above timings are based on the looped macros. If space permits,
;       approximately 65-69 clocks can be saved by using the unrolled macros.
;

;**********************************************************************************************
;**********************************************************************************************

;       16/16 Bit Division Macros


UDIV1616L       macro

;       restore = 23 clks,  nonrestore = 17 clks

;       Max Timing:     2+15*23+22 = 369 clks

;       Min Timing:     2+15*17+16 = 273 clks

;       PM: 24                                  DM: 7

                MOVLW           D'16'
                MOVWF           LOOPCOUNT

LOOPU1616       RLF             AARGB0,W
                RLF             REMB1, F
                RLF             REMB0, F
                MOVF            BARGB1,W
                SUBWF           REMB1, F
                MOVF            BARGB0,W
                BTFSS           _C
                INCFSZ          BARGB0,W
                SUBWF           REMB0, F

                BTFSC           _C
                GOTO            UOK66LL         
                MOVF            BARGB1,W
                ADDWF           REMB1, F
                MOVF            BARGB0,W
                BTFSC           _C
                INCFSZ          BARGB0,W
                ADDWF           REMB0, F

                BCF             _C

UOK66LL RLF             AARGB1, F
                RLF             AARGB0, F

                DECFSZ          LOOPCOUNT, F
                GOTO            LOOPU1616

                endm

;**********************************************************************************************
;**********************************************************************************************
        
;       16/16 Bit Unsigned Fixed Point Divide 16/16 -> 16.16

;       Input:  16 bit unsigned fixed point dividend in AARGB0, AARGB1
;               16 bit unsigned fixed point divisor in BARGB0, BARGB1

;       Use:    CALL    FXD1616U

;       Output: 16 bit unsigned fixed point quotient in AARGB0, AARGB1
;               16 bit unsigned fixed point remainder in REMB0, REMB1

;       Result: AARG, REM  <--  AARG / BARG

;       Max Timing:     2+369+2 = 373 clks

;       Min Timing:     2+273+2 = 277 clks

;       PM: 2+24+1 = 27         DM: 7

FXD1616U        CLRF            REMB0
                CLRF            REMB1

                UDIV1616L

                RETLW           0x00

;**********************************************************************************************
;**********************************************************************************************

		; SERIAL.ASM


serial_Delay:
		MOVWF	serialDelayCount	; ulozit W do delay pocitadla
		NOP				; Helps with high speed	

serial_Delay_Loop:				; odecitaci smycka
		DECFSZ	serialDelayCount,F	; odecist
		GOTO	serial_Delay_Loop	; jeste jsme neodcetli vse
		RETURN				; odecteno, Return


serial_Init:
		BSF	STATUS,RP0		; Select page 1 (80H - FFH)
		BCF	TRISA,SERIAL_OUT	; Nastavit vystup
		BCF	STATUS,RP0		; Select page 0 (00H - 7FH)
		BCF	PORTA,SERIAL_OUT	; Vystup nastavit do LOW
		MOVLW	EE_DELAY_TIME		; adresa RS-232 delay bytu v EEPROM
		CALL	EEPROM_Read_Byte	; nacist z EEPROM hodnotu
		MOVWF	delay_time		; a ulozit ji sem
						
		RETURN				; Return from subroutine


serial_SEND_byte:				; Odeslat znak ve W
serial_Transmit:				; odesle byte po RS232
		MOVWF	serialData		; ulozit byte pro odeslani
		MOVLW	00AH			; 8 bitu 1 stop a 1 start bit
		MOVWF	serialCount		;
		BCF	STATUS,C		; Clear carry flag

serial_Transmit_Loop:
		BCF	PORTA,SERIAL_OUT	; zaciname s LOW na vystupu
		BTFSS	STATUS,C		; Skip if carry set
		BSF	PORTA,SERIAL_OUT	; kdyz posilame 0, poslat HIGH
		MOVF	delay_time, W		; rychlost RS232
		CALL	serial_Delay		; cekat dobu bitu
		BSF	STATUS,C		; Set carry flag
		RRF	serialData,F		; odrotovat bity odesilaneho 
		DECFSZ	serialCount,F		; bytu, odecist odeslany bit
		GOTO	serial_Transmit_Loop	; jeste neodeslany vsechny
		RETURN				; odeslano, Return

serialTXcr:
		MOVLW	00DH			; Odeslat CR
		GOTO	serial_SEND_byte		
		RETURN			

serialTXlf:
		MOVLW	00AH			; Odeslat LF
		GOTO	serial_SEND_byte
		RETURN


serialTXsp:	
		MOVLW	020H			; Odeslat mezeru
		GOTO	serial_SEND_byte
		RETURN

serial_Receive:					;
		BCF	serialData,00		; pripravime prijaty bit jako 0
		BTFSS	PORTA,SERIAL_IN		; pokud neprijimame data
		RETLW	000H			; tak navrat
		MOVLW	009H			; jinak cteme
		MOVWF	serialCount		; 9 bitu
		MOVF	delay_time, W		; rychlost RS232
		CALL	serial_Delay		; cekat delku bitu

serial_Receive_Loop:				; 
		BCF	STATUS,C		; Clear carry flag
		BTFSS	PORTA,SERIAL_IN		; pokud prichazi 0
		BSF	STATUS,C		; Set carry flag
		RRF	serialData,F		; odrotovat serial data bits
		MOVF	delay_time, W		; rychlost RS232
		CALL	serial_Delay		; cekat delku bitu
		NOP				; keep timing w/ transmit
		DECFSZ	serialCount,F		; mame vsechny bity?
		GOTO	serial_Receive_Loop	; jeste ne, tak dalsi
		MOVF	serialData,W		; prijaty byte do W

serial_Receive_Stop:				;
		BTFSC	PORTA,SERIAL_IN		; cekat na 0 na serial in
		GOTO	serial_Receive_Stop	; je tam stale 1 (stop bit)
		BSF	serialData,00		; bit 0 nastavit na 1
		RETURN				; Return from subroutine


serial_Receive_BYTE:				; prijme byte z RS232 portu
		CALL	serial_Receive		; nacist byte
		BTFSS	serialData,00		; nacten? 
		GOTO	serial_Receive_BYTE	; ne, data zadna neprichazeji
		RETURN				; Byte nacten, Return
		
		; EEPROM.ASM

EEPROM_Read_Byte:			; cte byte z adresy W EEPROM
		ANDLW	03FH		; odmaskovat bity 6 a 7 pro 64Bytu RAM
		;ANDLW	07FH		; odmaskovat bity 7 pro 128Bytu RAM
		MOVWF	EEADR		; zapsat adresu, ze ktere budeme cist
		BSF	STATUS,RP0	; Select page 1 (80H - FFH)
		BSF	EECON1,RD	; bude se cist z EEPROM 
		BCF	STATUS,RP0	; Select page 0 (00H - 7FH)
		MOVF	EEDATA,W	; a nacteme byte
		RETURN			; Return from subroutine


EEPROM_Write_Byte:			; zapise byte na adresu W EEPROM
		ANDLW	03FH		; odmaskovat bity 6 a 7 pro 64Bytu RAM
		;ANDLW	07FH		; odmaskovat bit 7 pro 128Bytu RAM
		MOVWF	EEADR		; zapsat adresu, kam budeme zapisovat
		BSF	STATUS,RP0	; Select page 1 (80H - FFH)
		BCF	INTCON,GIE	; Generalni zakaz preruseni
		BSF	EECON1,WREN	; Povolit EEPROM write
		MOVLW	055H		; REQUIRED WRITE SEQUENCE START
		MOVWF	EECON2		; Write 55h
		MOVLW	0AAH		; 
		MOVWF	EECON2		; Write AAh
		BSF	EECON1,WR	; Set WR bit, zapis spusten
		BCF	EECON1,WREN	; Zakazat EEPROM write
		BSF	INTCON,GIE	; Preruseni opet povolit

EEPROM_Write_Complete_Loop:		; ceka az se zapise do EEPROM
		BTFSC	EECON1,WR	; uz se byte zapsal?
		GOTO	EEPROM_Write_Complete_Loop	; jeste ne, cekat
		BCF	STATUS,RP0	; zapsano, Select page 0 (00H - 7FH)
		RETURN			; Return from subroutine

EE_Send_232:				; Posle po RS232 data z EEPROM
		CALL	EEPROM_Read_Byte	; nacist z EEPROM offset
		MOVWF	temp0			; a ulozit ho sem

EE_Send_232_Loop:			; Posle po RS232 data z EEPROM
		MOVF	temp0,W		; od adresy v 'temp0' do 00H bytu
		CALL	EEPROM_Read_Byte	; zjistit hodnotu
		IORLW	000H			; je to binarni nula?
		BTFSC	STATUS,Z		; Skip if not zero
		RETURN			; Je, vse poslano, navrat
		CALL	serial_SEND_byte		; poslat byte po RS232
		INCF	temp0,F			; posun na dalsi byte
		GOTO	EE_Send_232_Loop	; a zpracovat ho


Main_Loop:				; Hlavni smycka Programu
		BTFSC	PORTA,KONFIG_IN		; 1=merime nebo 0=konfigurujeme ?
		GOTO	Start_Mereni	; Merime
		CALL	serial_Receive	; cist z RS232
		BTFSS	serialData,00	; prislo neco?
		MOVWF	temp0		; prislo, ulozit to sem
		XORLW	01BH		; je to ESC ?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	Serial_Receive_ESC	; prisel ESC, co prijde dal?
		GOTO	Main_Loop	; bylo to neco jineho, ignorovat to

Serial_Receive_ESC:			; prislo ESC, zpracovavame asi prikaz
		CALL	serial_Receive_BYTE	; cist z RS232
		MOVWF	temp0		; ulozit to sem
		XORLW	01BH		; prislo ESC ?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	Serial_Receive_ESC	; prislo ESC, cekame na prikaz
		MOVF	temp0,W		; ESC to nebylo, zjistit prikaz
		XORLW	044H		; prislo D ?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	Write_Config_to_EEPROM	; 'D'=zapis konfigurace
		MOVF	temp0,W		; obnovit prijaty znak
		XORLW	04FH		; prislo O ?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	Write_Pos_to_EEPROM	; 'O'=zapis posit. retezce
		MOVF	temp0,W		; obnovit prijaty znak
		XORLW	056H		; prislo V ?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	Send_Version	; 'V'=poslat verzi FirmWare
		MOVF	temp0,W		; ESC to nebylo, zjistit prikaz
		XORLW	050H		; prislo P ?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	Send_Pos_from_EEPROM	; 'P'=precteni posit. retezce
		MOVF	temp0,W		; obnovit prijaty znak
		XORLW	055H		; prislo U ?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	Send_Config_from_EEPROM	; 'U'=precteni konfigurace
		MOVF	temp0,W		; obnovit prijaty znak
		XORLW	04DH		; prislo M ?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	Start_Mereni	; 'M'=1x zmerit a poslat
		GOTO	Main_Loop	; bylo to neco jineho, ignorovat to

Write_Pos_to_EEPROM:			; zapise data z RS232 do EEPROM
		MOVLW	014H		; delka retezce je 20d
		MOVWF	temp1		; a ulozit sem
		MOVLW	EE_WX_POS	; offset WX souradnic 
		CALL	EEPROM_Read_Byte	; nacist
		MOVWF	temp0		; a ulozit sem
		GOTO	Write_Config_to_EEPROM_Loop	; a zapsat

Send_Pos_from_EEPROM:			; zapise data z RS232 do EEPROM
		MOVLW	014H		; delka retezce je 20d
		MOVWF	temp1		; a ulozit sem
		CALL	serial_SEND_byte	; a poslat ji jako prvni
		MOVLW	EE_WX_POS	; offset WX souradnic 
		CALL	EEPROM_Read_Byte	; nacist
		MOVWF	temp0		; a ulozit sem
		GOTO	Send_Config_from_EEPROM_Loop	; a precist

Write_Config_to_EEPROM:			; zapise data z RS232 do EEPROM
		CLRF	temp0		; vynulovat ukazovatko
		CLRF	temp1		; vynulovat ukazovatko

Write_Config_to_EEPROM_Loop:		; smycka zapisu do EEPROM
		CALL	serial_Receive_BYTE	; prijmout byte z RS232
		;BSF	STATUS,RP0	; Select page 1 (80H - FFH)
		MOVWF	EEDATA		; ulozit ho sem
		;BCF	STATUS,RP0	; Select page 0 (00H - 7FH)
		MOVF	temp0,W		; sem dat adresu bytu v EEPROM
		CALL	EEPROM_Write_Byte	; zapsat byte do EEPROM
		DECF	temp1,F		; snizit pocet bytu pro zapis
		BTFSC	STATUS,Z 	; zapsany vsechny pozadovane byty?
		GOTO	Main_Loop	; ano, jedeme dokolecka
		INCF	temp0,F		; posun na dalsi byte
		BTFSS	temp0,EEPROM_size_bit	; bylo jich uz 64/128?
		GOTO	Write_Config_to_EEPROM_Loop	; jeste ne, dalsi
		GOTO	Main_Loop	; zapsano, jedeme dokolecka

Send_Config_from_EEPROM:		; nacte data z EEPROM a posle na RS232
		MOVLW	EE_LEN		; delka EEPROM
		CALL	serial_SEND_byte	; a poslat ji jako prvni
		CLRF	temp0		; vynulovat ukazovatko
		CLRF	temp1		; vynulovat ukazovatko

Send_Config_from_EEPROM_Loop:		; smycka cteni z EEPROM
		MOVF	temp0,W		; vzit adresu bytu v EEPROM
		CALL	EEPROM_Read_Byte	; nacist ho
		CALL	serial_SEND_byte	; a poslat po RS232
		DECF	temp1,F		; snizit pocet bytu pro zapis
		BTFSC	STATUS,Z 	; zapsany vsechny pozadovane byty?
		GOTO	Main_Loop	; ano, jedeme dokolecka
		INCF	temp0,F		; posun na dalsi byte
		BTFSS	temp0,EEPROM_size_bit	; bylo jich uz 64/128?
		GOTO	Send_Config_from_EEPROM_Loop	; jeste ne, dalsi
		GOTO	Main_Loop	; vse odeslano, jedeme dokolecka

Send_Version:				; odesle po startu text verze 
		CLRF	temp0		; po RS232, vynulovat temp0

Send_Version_serial:			; po RS232 odesle text verze FW
		MOVF	temp0,W		; offset bytu verze
		CALL	Version_Data	; ziskat byte verze
		IORLW	000H		; je to binarni nula?
		BTFSC	STATUS,Z	; Skip if not zero
		GOTO	Send_Version_END	; je, cely text byl poslan
		CALL	serial_SEND_byte	; poslat byte
		INCF	temp0,F		; posun na dalsi byte
		GOTO	Send_Version_serial	; a zpracovat ho

Send_Version_END:				; jeste cr+lf
		CALL	serialTXcr		;
		CALL	serialTXlf		;
		GOTO	Main_Loop	; je, cely text byl poslan

		; Pojistky procesoru
		
		ORG	2007H

		DATA	3FF1H

		; EEPROM procesoru

EEPROM_size_bit	EQU	006H		; Bitik preteceni EEPROM
					; 64B=> 06h, 128B=>07h
EE_LEN		EQU	040H		; velikost EEPROM

		ORG	2100H

RESERVED_0	EQU	000H		; 
RESERVED_1	EQU	001H		; 
RESERVED_2	EQU	002H		; 
RESERVED_3	EQU	003H		; 
RESERVED_4	EQU	004H		; 
RESERVED_5	EQU	005H		; 
RESERVED_6	EQU	006H		; 
RESERVED_7	EQU	007H		; 
EE_DELAY_TIME	EQU	008H		; Adresa RS-232 DELAY time v EEPROM
EE_WX_POS	EQU	009H		; Adresa WX souradnic
EE_WX_COMMENT	EQU	00AH		; Adresa komentare WX ramce

	DATA	0000H			; 
		DATA	0000H		; 
		DATA	0000H		; 
		DATA	0000H		; 
		DATA	0000H		; 
		DATA	0000H		; 
		DATA	0000H		; 
		DATA	0000H		; 
	; RS-232 delay time should be: ((ClockSpeed/4/SerialSpeed)-14)/3
	; RS-232 9k6 @10MHz => 52h   9k6 @4MHz => 1Eh
		DATA	001DH		; Testovano 9k6 @4MHz=1Dh @10MHz=50h
		DATA	000BH		; Adresa zacatku WX souradnic
		DATA	002CH		; Adresa zacatku WX komentare

		DATA	0021H		; '!' = id WX bez timestampu 
		DATA	0035H		; latitude
		DATA	0030H		; degrees
		DATA	0030H		; latitude
		DATA	0033H		; minutes
	DATA	002EH			; '.'
		DATA	0030H		; latitude
		DATA	0030H		; hundrets
		DATA	004EH		; latitude N/S
		DATA	002FH		; Icon-Table
		DATA	0030H		; longitude
		DATA	0031H		; deg-
		DATA	0034H		; rees
		DATA	0032H		; longitude
		DATA	0037H		; minutes
		DATA	002EH		; '.'
		DATA	0030H		; longitude
		DATA	0030H		; hundreds
		DATA	0045H		; longitude W/E
		DATA	005FH		; Icon itself
		DATA	0030H		; '0'
	DATA	0030H			; '0'
		DATA	0030H		; '0'
		DATA	002FH		; '/'
		DATA	0030H		; '0'
		DATA	0030H		; '0'
		DATA	0030H		; '0'
		DATA	0067H		; 'g'
		DATA	0030H		; '0'
		DATA	0030H		; '0'
		DATA	0030H		; '0'
		DATA	0074H		; 't'
		DATA	0000H		;

		DATA	0058H		; Komentar: 'X' jako verze WX sw
		DATA	0050H		; 'P'	verze WX je vzdy PIC 
		DATA	0049H		; 'I'	verze WX je vzdy PIC
		DATA	0043H		; 'C'	verze WX je vzdy PIC
	DATA	0020H			; ' '	pak mezera a 
		DATA	0053H		; 'S'
		DATA	004DH		; 'M'
		DATA	0054H		; 'T'
		DATA	0020H		; ' '
		DATA	0074H		; 't'
		DATA	0065H		; 'e'
		DATA	006DH		; 'm'
		DATA	0070H		; 'p'
		DATA	003AH		; ':'
		DATA	0000H		;

		;!5003.45N/01430.58E_000/000g000tteplotaFXPICteplotaCC

		END				;
